#
#   Copyright (c) 2003, Joerg Raedler, <jr@j-raedler.de>
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#   Neither the name of the author nor the names of its contributors may
#   be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#   ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#   POSSIBILITY OF SUCH DAMAGE.


from cFuGrep import *

try:
    bool(1)
except:
    False, True = 0, 1
    def bool(o):
        if o: return True
        else: return False


class XPattern(Pattern):
    """Extended FuGrep.Pattern with some extra methods"""

    def wordlist(self, text, greedy=1):
        """returns the list of words matching the text"""
        res = self.grep(text, greedy)
        return res and [text[a:b] for a,b in res] or []

    def replace(self, text, rep='', greedy=1):
        """replaces all matching words with a (possibly empty) string"""
        res = self.grep(text, greedy)
        if not res:
            return text
        l = [text[:res[0][0]]]
        for i in range(len(res)-1):
            l.append(text[res[i][1]:res[i+1][0]])
        l.append(text[res[-1][1]:])
        return rep.join(l)

    def lgrep(self, tlist, greedy=1):
        """apply grep to a list of strings, return list of results"""
        return [self.grep(text, greedy) for text in tlist]

    def contains(self, text):
        """return boolean value if pattern is contained in text"""
        return bool(self.grep(text))

    def lcontains(self, tlist):
        """apply contains to a list of strings, return a list of boolean values"""
        return [bool(self.grep(text)) for text in tlist]

    def count(self, text):
        """return the number of matches"""
        return len(self.grep(text))

    def grepSorted(self, text, greedy):
        tmp = [(1.0-similarity(text[a:b], self.pattern), (a,b)) for a,b in self.grep(text, greedy)]
        tmp.sort()
        return [y for x,y in tmp]


def sortBySimilarity(word, wlist):
    tmp = [(1.0-similarity(word, wlist[i]), i) for i in range(len(wlist))]
    tmp.sort()
    return [wlist[j] for i,j in tmp]


def __createPattern(txt, nerr):
    """rebuild Pattern from pickled data"""
    return Pattern(txt, nerr)

def __reducePattern(p):
    """return pickle data for Pattern"""
    return (__createPattern, (p.text, p.nErrors))

import copy_reg
copy_reg.constructor(__createPattern)
copy_reg.pickle(Pattern, __reducePattern)
del copy_reg


if __name__ == '__main__':

    def info():
        import cFuGrep as x
        print x.version
        print x._author
        print x._homepage
        print x.__doc__
        print x.Pattern.__doc__
        print x.Pattern("foo").grep.__doc__

    def test0():
        txt = "Shit, what the hell is wrong with the word 'shit'?"
        pat = XPattern('shit', 1)
        print pat.replace(txt, '|BEEP|')
        print "The text contains the word %dx" % pat.count(txt)

    def test1():
        res = lambda s, l: l and [s[a:b] for a,b in l] or []
        p = XPattern('dax', 1)
        txt = "rix rux rax dax fax fox lax lix"
        print res(txt, p.grep(txt, 1))
        print res(txt, p.grepSorted(txt, 1))

    def test2():
        word = 'dax'
        wlist = "rix rux rax dax fax fox lax lix".split()
        print sortBySimilarity(word, wlist)

    info()
    test0()
    test1()
    test2()
